/**
 * DevExtreme (exporter/exceljs/exportDataGrid.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});

function exportDataGrid(options) {
    var dataGrid = options.dataGrid,
        worksheet = options.worksheet,
        _options$topLeftCell = options.topLeftCell,
        topLeftCell = void 0 === _options$topLeftCell ? {
            row: 0,
            column: 0
        } : _options$topLeftCell;
    var result = {
        from: {
            row: topLeftCell.row,
            column: topLeftCell.column
        },
        to: {
            row: 0,
            column: 0
        }
    };
    var columns = dataGrid.getVisibleColumns().filter(function(item) {
        return item.allowExporting
    });
    if (false !== dataGrid.option("showColumnHeaders")) {
        var headerRow = worksheet.addRow();
        for (var i = 0; i < columns.length; i++) {
            headerRow.getCell(i + 1).value = columns[i].caption
        }
    }
    if (options && true === options.excelFilterEnabled) {
        worksheet.autoFilter = {
            from: {
                row: 1,
                column: 1
            },
            to: {
                row: 1,
                column: columns.length
            }
        };
        worksheet.views = [{
            state: "frozen",
            ySplit: 1
        }]
    }
    return new Promise(function(resolve) {
        dataGrid.getController("data").loadAll().then(function(items) {
            for (var _i = 0; _i < items.length; _i++) {
                var dataRow = worksheet.addRow();
                for (var j = 0; j < items[_i].values.length; j++) {
                    dataRow.getCell(j + 1).value = items[_i].values[j]
                }
            }
            resolve(result)
        })
    })
}
exports.exportDataGrid = exportDataGrid;
