/**
 * DevExtreme (file_provider/file_provider.array.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _typeof = "function" === typeof Symbol && "symbol" === typeof Symbol.iterator ? function(obj) {
    return typeof obj
} : function(obj) {
    return obj && "function" === typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj
};
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _iterator = require("../core/utils/iterator");
var _file_provider = require("./file_provider");

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var ArrayFileProvider = function(_FileProvider) {
    _inherits(ArrayFileProvider, _FileProvider);

    function ArrayFileProvider(data) {
        _classCallCheck(this, ArrayFileProvider);
        var _this = _possibleConstructorReturn(this, (ArrayFileProvider.__proto__ || Object.getPrototypeOf(ArrayFileProvider)).call(this));
        _this._data = data || [];
        return _this
    }
    _createClass(ArrayFileProvider, [{
        key: "getItems",
        value: function(path, itemType) {
            return this._getItems(path, itemType)
        }
    }, {
        key: "renameItem",
        value: function(item, name) {
            item.dataItem.name = name
        }
    }, {
        key: "createFolder",
        value: function(parentFolder, name) {
            var newItem = {
                name: name,
                isFolder: true
            };
            var array = this._getChildrenArray(parentFolder.dataItem);
            array.push(newItem)
        }
    }, {
        key: "deleteItems",
        value: function(items) {
            var _this2 = this;
            (0, _iterator.each)(items, function(_, item) {
                return _this2._deleteItem(item)
            })
        }
    }, {
        key: "moveItems",
        value: function(items, destinationFolder) {
            var _this3 = this;
            var array = this._getChildrenArray(destinationFolder.dataItem);
            (0, _iterator.each)(items, function(_, item) {
                _this3._deleteItem(item);
                array.push(item.dataItem)
            })
        }
    }, {
        key: "copyItems",
        value: function(items, destinationFolder) {
            var _this4 = this;
            var array = this._getChildrenArray(destinationFolder.dataItem);
            (0, _iterator.each)(items, function(_, item) {
                var copiedItem = _this4._createCopy(item.dataItem);
                array.push(copiedItem)
            })
        }
    }, {
        key: "_createCopy",
        value: function(dataItem) {
            var _this5 = this;
            var result = {
                name: dataItem.name,
                isFolder: dataItem.isFolder
            };
            if (dataItem.children) {
                result.children = [];
                (0, _iterator.each)(dataItem.children, function(_, childItem) {
                    var childCopy = _this5._createCopy(childItem);
                    result.children.push(childCopy)
                })
            }
            return result
        }
    }, {
        key: "_deleteItem",
        value: function(item) {
            var array = this._data;
            if ("" !== item.parentPath) {
                var entry = this._findItem(item.parentPath);
                array = entry.children
            }
            var index = array.indexOf(item.dataItem);
            array.splice(index, 1)
        }
    }, {
        key: "_getChildrenArray",
        value: function(dataItem) {
            var array = null;
            if (!dataItem) {
                array = this._data
            } else {
                if (!dataItem.children) {
                    dataItem.children = []
                }
                array = dataItem.children
            }
            return array
        }
    }, {
        key: "_getItems",
        value: function(path, itemType) {
            if ("" === path) {
                return this._getItemsInternal(path, this._data, itemType)
            }
            var folderEntry = this._findItem(path);
            var entries = folderEntry && folderEntry.children || [];
            return this._getItemsInternal(path, entries, itemType)
        }
    }, {
        key: "_findItem",
        value: function(path) {
            if ("" === path) {
                return null
            }
            var result = null;
            var data = this._data;
            var parts = path.split("/");
            var _loop = function(i) {
                var part = parts[i];
                result = data.filter(function(entry) {
                    return entry.isFolder && entry.name === part
                })[0];
                if (result) {
                    if (result.children) {
                        data = result.children
                    } else {
                        if (i !== parts.length - 1) {
                            return {
                                v: null
                            }
                        }
                    }
                } else {
                    return {
                        v: null
                    }
                }
            };
            for (var i = 0; i < parts.length; i++) {
                var _ret = _loop(i);
                if ("object" === ("undefined" === typeof _ret ? "undefined" : _typeof(_ret))) {
                    return _ret.v
                }
            }
            return result
        }
    }, {
        key: "_getItemsInternal",
        value: function(path, data, itemType) {
            var useFolders = "folder" === itemType;
            var result = [];
            (0, _iterator.each)(data, function(_, entry) {
                var isFolder = !!entry.isFolder;
                if (!itemType || isFolder === useFolders) {
                    var item = new _file_provider.FileManagerItem(path, entry.name, isFolder);
                    item.length = void 0 !== entry.length ? entry.length : 0;
                    item.lastWriteTime = void 0 !== entry.lastWriteTime ? entry.lastWriteTime : new Date;
                    item.thumbnail = entry.thumbnail;
                    item.dataItem = entry;
                    result.push(item)
                }
            });
            return result
        }
    }]);
    return ArrayFileProvider
}(_file_provider.FileProvider);
module.exports = ArrayFileProvider;
