/**
 * DevExtreme (file_provider/file_provider.webapi.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _ajax = require("../core/utils/ajax");
var _ajax2 = _interopRequireDefault(_ajax);
var _common = require("../core/utils/common");
var _guid = require("../core/guid");
var _guid2 = _interopRequireDefault(_guid);
var _window = require("../core/utils/window");
var _window2 = _interopRequireDefault(_window);
var _iterator = require("../core/utils/iterator");
var _file_provider = require("./file_provider");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var window = _window2.default.getWindow();
var FILE_CHUNK_BLOB_NAME = "chunk";
var FILE_CHUNK_META_DATA_NAME = "chunkMetadata";
var WebAPIFileProvider = function(_FileProvider) {
    _inherits(WebAPIFileProvider, _FileProvider);

    function WebAPIFileProvider(options) {
        _classCallCheck(this, WebAPIFileProvider);
        var _this = _possibleConstructorReturn(this, (WebAPIFileProvider.__proto__ || Object.getPrototypeOf(WebAPIFileProvider)).call(this));
        _this._options = options;
        return _this
    }
    _createClass(WebAPIFileProvider, [{
        key: "getItems",
        value: function(path, itemType) {
            return this._getItems(path, itemType)
        }
    }, {
        key: "renameItem",
        value: function(item, name) {
            return this._executeRequest(this._options.renameUrl, {
                id: item.relativeName,
                newName: name
            })
        }
    }, {
        key: "createFolder",
        value: function(parentFolder, name) {
            return this._executeRequest(this._options.createFolderUrl, {
                parentId: parentFolder.relativeName,
                name: name
            })
        }
    }, {
        key: "deleteItems",
        value: function(items) {
            var _this2 = this;
            return items.map(function(item) {
                return _this2._executeRequest(_this2._options.deleteUrl, {
                    id: item.relativeName
                })
            })
        }
    }, {
        key: "moveItems",
        value: function(items, destinationFolder) {
            var _this3 = this;
            return items.map(function(item) {
                return _this3._executeRequest(_this3._options.moveUrl, {
                    sourceId: item.relativeName,
                    destinationId: destinationFolder.relativeName + "/" + item.name
                })
            })
        }
    }, {
        key: "copyItems",
        value: function(items, destinationFolder) {
            var _this4 = this;
            return items.map(function(item) {
                return _this4._executeRequest(_this4._options.copyUrl, {
                    sourceId: item.relativeName,
                    destinationId: destinationFolder.relativeName
                })
            })
        }
    }, {
        key: "initiateFileUpload",
        value: function(uploadInfo) {
            uploadInfo.customData.uploadId = new _guid2.default
        }
    }, {
        key: "uploadFileChunk",
        value: function(uploadInfo, chunk) {
            var params = {
                destinationId: uploadInfo.destinationFolder.relativeName
            };
            var url = this._options.uploadChunkUrl + "?" + this._getQueryString(params);
            var formData = new window.FormData;
            formData.append(FILE_CHUNK_BLOB_NAME, chunk.blob);
            formData.append(FILE_CHUNK_META_DATA_NAME, JSON.stringify({
                UploadId: uploadInfo.customData.uploadId,
                FileName: uploadInfo.file.name,
                Index: chunk.index,
                TotalCount: uploadInfo.totalChunkCount,
                FileSize: uploadInfo.file.size
            }));
            return _ajax2.default.sendRequest({
                url: url,
                method: "POST",
                dataType: "text",
                data: formData,
                upload: {
                    onprogress: _common.noop,
                    onloadstart: _common.noop,
                    onabort: _common.noop
                },
                cache: false
            })
        }
    }, {
        key: "abortFileUpload",
        value: function(uploadInfo) {
            return this._executeRequest(this._options.abortUploadUrl, {
                uploadId: uploadInfo.customData.uploadId
            })
        }
    }, {
        key: "_getItems",
        value: function(path, itemType) {
            var _this5 = this;
            return this._getEntriesByPath(path).then(function(entries) {
                return _this5._convertEntriesToItems(entries, path, itemType)
            })
        }
    }, {
        key: "_getItemsIds",
        value: function(items) {
            return items.map(function(it) {
                return it.relativeName
            })
        }
    }, {
        key: "_getEntriesByPath",
        value: function(path) {
            return this._executeRequest(this._options.loadUrl, {
                parentId: path
            }, true)
        }
    }, {
        key: "_executeRequest",
        value: function(url, params, jsonResult) {
            var queryString = this._getQueryString(params);
            var finalUrl = url + "?" + queryString;
            var dataType = jsonResult ? "json" : "text";
            return _ajax2.default.sendRequest({
                url: finalUrl,
                dataType: dataType,
                cache: false
            })
        }
    }, {
        key: "_getQueryString",
        value: function(params) {
            var pairs = [];
            var keys = Object.keys(params);
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                var value = params[key];
                if (void 0 === value) {
                    continue
                }
                if (null === value) {
                    value = ""
                }
                if (Array.isArray(value)) {
                    this._processQueryStringArrayParam(key, value, pairs)
                } else {
                    var pair = this._getQueryStringPair(key, value);
                    pairs.push(pair)
                }
            }
            return pairs.join("&")
        }
    }, {
        key: "_processQueryStringArrayParam",
        value: function(key, array, pairs) {
            var _this6 = this;
            (0, _iterator.each)(array, function(_, item) {
                var pair = _this6._getQueryStringPair(key, item);
                pairs.push(pair)
            })
        }
    }, {
        key: "_getQueryStringPair",
        value: function(key, value) {
            return encodeURIComponent(key) + "=" + encodeURIComponent(value)
        }
    }, {
        key: "_convertEntriesToItems",
        value: function(entries, path, itemType) {
            var useFolders = "folder" === itemType;
            var result = [];
            (0, _iterator.each)(entries, function(_, entry) {
                var isFolder = !!entry.isFolder;
                if (!itemType || isFolder === useFolders) {
                    var item = new _file_provider.FileManagerItem(path, entry.name, isFolder);
                    item.length = entry.size || 0;
                    item.lastWriteTime = entry.lastWriteTime;
                    result.push(item)
                }
            });
            return result
        }
    }]);
    return WebAPIFileProvider
}(_file_provider.FileProvider);
module.exports = WebAPIFileProvider;
