/**
 * DevExtreme (ui/drop_down_button.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _renderer = require("../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _ui = require("./widget/ui.widget");
var _ui2 = _interopRequireDefault(_ui);
var _component_registrator = require("../core/component_registrator");
var _component_registrator2 = _interopRequireDefault(_component_registrator);
var _button_group = require("./button_group");
var _button_group2 = _interopRequireDefault(_button_group);
var _popup = require("./popup");
var _popup2 = _interopRequireDefault(_popup);
var _list = require("./list");
var _list2 = _interopRequireDefault(_list);
var _data = require("../core/utils/data");
var _data_helper = require("../data_helper");
var _data_helper2 = _interopRequireDefault(_data_helper);
var _data_source = require("../data/data_source/data_source");
var _array_store = require("../data/array_store");
var _array_store2 = _interopRequireDefault(_array_store);
var _deferred = require("../core/utils/deferred");
var _extend = require("../core/utils/extend");
var _type = require("../core/utils/type");
var _common = require("../core/utils/common");
var _message = require("../localization/message");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var DROP_DOWN_BUTTON_CLASS = "dx-dropdownbutton";
var DROP_DOWN_BUTTON_CONTENT = "dx-dropdownbutton-content";
var DROP_DOWN_BUTTON_ACTION_CLASS = "dx-dropdownbutton-action";
var DROP_DOWN_BUTTON_TOGGLE_CLASS = "dx-dropdownbutton-toggle";
var DropDownButton = _ui2.default.inherit({
    _getDefaultOptions: function() {
        return (0, _extend.extend)(this.callBase(), {
            itemTemplate: "item",
            keyExpr: "this",
            displayExpr: "this",
            selectedItem: null,
            deferRendering: true,
            noDataText: (0, _message.format)("dxCollectionWidget-noDataText"),
            showSelectedItem: true,
            onActionButtonClick: null,
            onSelectionChanged: null,
            items: [],
            dataSource: null,
            dropDownOptions: {},
            grouped: false,
            groupTemplate: "group",
            buttonGroupOptions: {}
        })
    },
    _setOptionsByReference: function() {
        this.callBase();
        (0, _extend.extend)(this._optionsByReference, {
            selectedItem: true
        })
    },
    _init: function() {
        this.callBase();
        this._createItemClickAction();
        this._createActionClickAction();
        this._createSelectionChangedAction();
        this._compileKeyGetter();
        this._compileDisplayGetter();
        this._initDataSource();
        this._itemsToDataSource();
        this._initInnerOptionCache("buttonGroupOptions");
        this._initInnerOptionCache("dropDownOptions")
    },
    _itemsToDataSource: function() {
        if (!this.option("dataSource")) {
            this._dataSource = new _data_source.DataSource({
                store: new _array_store2.default(this.option("items")),
                pageSize: 0
            })
        }
    },
    _compileKeyGetter: function() {
        this._keyGetter = (0, _data.compileGetter)(this.option("keyExpr"))
    },
    _compileDisplayGetter: function() {
        this._displayGetter = (0, _data.compileGetter)(this.option("displayExpr"))
    },
    _initMarkup: function() {
        var _this = this;
        this.$element().addClass(DROP_DOWN_BUTTON_CLASS);
        this._renderButtonGroup();
        this._loadSelectedItem().done(function(selectedItem) {
            _this._setOptionSilent("selectedItem", selectedItem);
            _this._showSelectedItem()
        });
        if (!this.option("deferRendering")) {
            this._renderPopup()
        }
    },
    _loadSelectedItem: function() {
        var d = new _deferred.Deferred;
        if (this._list) {
            return d.resolve(this._list.option("selectedItem"))
        }
        var selectedItem = this.option("selectedItem");
        var selectedItemKey = this._keyGetter(selectedItem);
        this._loadSingle(this.option("keyExpr"), selectedItemKey).done(d.resolve).fail(function() {
            d.resolve(selectedItem)
        });
        return d.promise()
    },
    _createActionClickAction: function() {
        this._actionClickAction = this._createActionByOption("onActionButtonClick")
    },
    _createSelectionChangedAction: function() {
        this._selectionChangedAction = this._createActionByOption("onSelectionChanged")
    },
    _createItemClickAction: function() {
        this._itemClickAction = this._createActionByOption("onItemClick")
    },
    _fireSelectionChangedAction: function(_ref) {
        var previousValue = _ref.previousValue,
            value = _ref.value;
        this._selectionChangedAction({
            oldSelectedItem: previousValue,
            selectedItem: value
        })
    },
    _fireItemClickAction: function(_ref2) {
        var event = _ref2.event,
            itemElement = _ref2.itemElement,
            itemData = _ref2.itemData;
        return this._itemClickAction({
            event: event,
            itemElement: itemElement,
            itemData: this._actionItem || itemData
        })
    },
    _actionButtonConfig: function() {
        var _this2 = this;
        var defaultConfig = {
            onClick: function(_ref3) {
                var event = _ref3.event;
                _this2._actionClickAction({
                    event: event,
                    selectedItem: _this2.option("selectedItem")
                })
            },
            elementAttr: {
                "class": DROP_DOWN_BUTTON_ACTION_CLASS
            }
        };
        var selectedItem = this.option("selectedItem");
        if ((0, _type.isPlainObject)(selectedItem)) {
            var displayValue = this._displayGetter(selectedItem);
            if (!(0, _type.isPlainObject)(displayValue)) {
                selectedItem.text = String((0, _common.ensureDefined)(displayValue, ""))
            }
        } else {
            selectedItem = {
                text: String((0, _common.ensureDefined)(selectedItem, ""))
            }
        }
        return (0, _extend.extend)({}, selectedItem, defaultConfig)
    },
    _buttonGroupOptions: function() {
        return (0, _extend.extend)({
            items: [this._actionButtonConfig(), {
                icon: "spindown",
                width: 24,
                elementAttr: {
                    "class": DROP_DOWN_BUTTON_TOGGLE_CLASS
                },
                onClick: this.toggle.bind(this, void 0)
            }],
            stylingMode: "outlined",
            selectionMode: "none"
        }, this._getInnerOptionsCache("buttonGroupOptions"))
    },
    _popupOptions: function() {
        var _this3 = this;
        return (0, _extend.extend)({
            deferRendering: this.option("deferRendering"),
            minWidth: 130,
            closeOnOutsideClick: true,
            showTitle: false,
            animation: {
                show: {
                    type: "fade",
                    duration: 0,
                    from: 0,
                    to: 1
                },
                hide: {
                    type: "fade",
                    duration: 400,
                    from: 1,
                    to: 0
                }
            },
            width: "auto",
            height: "auto",
            shading: false,
            position: {
                of: this._buttonGroup.element(),
                collision: "flipfit",
                my: "top right",
                at: "bottom right",
                offset: {
                    y: -1
                }
            },
            contentTemplate: function(content) {
                var $content = (0, _renderer2.default)(content);
                $content.addClass(DROP_DOWN_BUTTON_CONTENT);
                _this3._list = _this3._createComponent((0, _renderer2.default)("<div>"), _list2.default, _this3._listOptions());
                $content.append(_this3._list.$element())
            }
        }, this._getInnerOptionsCache("dropDownOptions"))
    },
    _listOptions: function() {
        var _this4 = this;
        return {
            selectionMode: "single",
            selectedItemKeys: [this._keyGetter(this.option("selectedItem"))],
            grouped: this.option("grouped"),
            keyExpr: this.option("keyExpr"),
            noDataText: this.option("noDataText"),
            displayExpr: this.option("displayExpr"),
            itemTemplate: this.option("itemTemplate"),
            tabIndex: null,
            items: this.option("items"),
            dataSource: this.option("dataSource"),
            onItemClick: function(e) {
                _this4.option("selectedItem", e.itemData);
                var actionResult = _this4._fireItemClickAction(e);
                if (false !== actionResult) {
                    _this4.toggle(false)
                }
            }
        }
    },
    _renderPopup: function() {
        var $popup = (0, _renderer2.default)("<div>");
        this.$element().append($popup);
        this._popup = this._createComponent($popup, _popup2.default, this._popupOptions());
        this._bindInnerWidgetOptions(this._popup, "dropDownOptions")
    },
    _renderButtonGroup: function() {
        var $buttonGroup = this._buttonGroup && this._buttonGroup.$element() || (0, _renderer2.default)("<div>");
        if (!this._buttonGroup) {
            this.$element().append($buttonGroup)
        }
        this._buttonGroup = this._createComponent($buttonGroup, _button_group2.default, this._buttonGroupOptions());
        this._bindInnerWidgetOptions(this._buttonGroup, "buttonGroupOptions")
    },
    toggle: function(visible) {
        this._popup || this._renderPopup();
        return this._popup.toggle(visible)
    },
    open: function() {
        return this.toggle(true)
    },
    close: function() {
        return this.toggle(false)
    },
    _setListOption: function(name, value) {
        this._list && this._list.option(name, value)
    },
    _showSelectedItem: function() {
        if (this.option("showSelectedItem")) {
            this._buttonGroup.option("items[0]", this._actionButtonConfig())
        }
    },
    _clean: function() {
        this._list && this._list.$element().remove();
        this._popup && this._popup.$element().remove()
    },
    _optionChanged: function(args) {
        var _this5 = this;
        var name = args.name,
            value = args.value;
        switch (args.name) {
            case "items":
            case "dataSource":
            case "itemTemplate":
            case "showSelectedItem":
                break;
            case "displayExpr":
                this._compileDisplayGetter();
                break;
            case "keyExpr":
                this._compileKeyGetter();
                break;
            case "buttonGroupOptions":
                this._innerOptionChanged(this._buttonGroup, args);
                break;
            case "dropDownOptions":
                this._innerOptionChanged(this._popup, args);
                break;
            case "grouped":
            case "noDataText":
            case "groupTemplate":
                this._setListOption(name, value);
                break;
            case "selectedItem":
                this._setListOption("selectedItemKeys", [this._keyGetter(value)]);
                this._loadSelectedItem().done(function(selectedItem) {
                    _this5._setOptionSilent("selectedItem", selectedItem);
                    if (_this5._keyGetter(args.previousValue) !== _this5._keyGetter(value)) {
                        _this5._showSelectedItem();
                        _this5._fireSelectionChangedAction(args)
                    }
                });
                break;
            case "onItemClick":
                this._createItemClickAction();
                break;
            case "onActionButtonClick":
                this._createActionClickAction();
                break;
            case "onSelectionChanged":
                this._createSelectionChangedAction();
                break;
            case "deferRendering":
                if (!value && !this._popup) {
                    this._renderPopup()
                }
                break;
            default:
                this.callBase(args)
        }
    }
}).include(_data_helper2.default);
(0, _component_registrator2.default)("dxDropDownButton", DropDownButton);
module.exports = DropDownButton;
module.exports.default = module.exports;
