/**
 * DevExtreme (ui/file_manager/ui.file_manager.files_tree_view.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _get = function get(object, property, receiver) {
    if (null === object) {
        object = Function.prototype
    }
    var desc = Object.getOwnPropertyDescriptor(object, property);
    if (void 0 === desc) {
        var parent = Object.getPrototypeOf(object);
        if (null === parent) {
            return
        } else {
            return get(parent, property, receiver)
        }
    } else {
        if ("value" in desc) {
            return desc.value
        } else {
            var getter = desc.get;
            if (void 0 === getter) {
                return
            }
            return getter.call(receiver)
        }
    }
};
var _events_engine = require("../../events/core/events_engine");
var _events_engine2 = _interopRequireDefault(_events_engine);
var _deferred = require("../../core/utils/deferred");
var _iterator = require("../../core/utils/iterator");
var _ui = require("../widget/ui.widget");
var _ui2 = _interopRequireDefault(_ui);
var _extend = require("../../core/utils/extend");
var _uiTree_view = require("../tree_view/ui.tree_view.search");
var _uiTree_view2 = _interopRequireDefault(_uiTree_view);
var _file_provider = require("../../file_provider/file_provider");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var PATH_SEPARATOR = "/";
var FileManagerFilesTreeView = function(_Widget) {
    _inherits(FileManagerFilesTreeView, _Widget);

    function FileManagerFilesTreeView() {
        _classCallCheck(this, FileManagerFilesTreeView);
        return _possibleConstructorReturn(this, (FileManagerFilesTreeView.__proto__ || Object.getPrototypeOf(FileManagerFilesTreeView)).apply(this, arguments))
    }
    _createClass(FileManagerFilesTreeView, [{
        key: "_initMarkup",
        value: function() {
            this._initCurrentPathState();
            this._filesTreeView = this._createComponent(this.$element(), _uiTree_view2.default, {
                dataStructure: "plain",
                rootValue: "",
                keyExpr: "relativeName",
                parentIdExpr: "parentPath",
                displayExpr: "name",
                createChildren: this._onFilesTreeViewCreateChildren.bind(this),
                onItemClick: this._onFilesTreeViewItemClick.bind(this),
                onItemExpanded: this._onFilesTreeViewItemExpanded.bind(this)
            });
            _events_engine2.default.on(this._filesTreeView.$element(), "click", this._raiseClick.bind(this))
        }
    }, {
        key: "_onFilesTreeViewCreateChildren",
        value: function(parent) {
            var _this2 = this;
            var parentItem = parent ? parent.itemData : null;
            var itemsGetter = this.option("getItems");
            var itemsResult = itemsGetter(parentItem);
            return (0, _deferred.when)(itemsResult).done(function(items) {
                return _this2._applyIconsToItems(items)
            })
        }
    }, {
        key: "_onFilesTreeViewItemClick",
        value: function(e) {
            this._changeCurrentFolder(e.itemData)
        }
    }, {
        key: "_changeCurrentFolder",
        value: function(folder) {
            var newPath = folder.relativeName;
            if (newPath !== this._currentPath) {
                this._currentPath = newPath;
                this._currentFolder = folder;
                this._raiseCurrentFolderChanged()
            }
        }
    }, {
        key: "_applyIconsToItems",
        value: function(items) {
            (0, _iterator.each)(items, function(_, item) {
                item.icon = "folder"
            })
        }
    }, {
        key: "_raiseCurrentFolderChanged",
        value: function() {
            this._raiseEvent("CurrentFolderChanged")
        }
    }, {
        key: "_raiseClick",
        value: function() {
            this._raiseEvent("Click")
        }
    }, {
        key: "_raiseEvent",
        value: function(eventName) {
            var handler = this.option("on" + eventName);
            if (handler) {
                handler()
            }
        }
    }, {
        key: "_initCurrentPathState",
        value: function() {
            this._currentPath = "";
            this._rootFolder = new _file_provider.FileManagerItem("", "");
            this._currentFolder = this._rootFolder;
            this._loadMap = {}
        }
    }, {
        key: "_ensurePathExpanded",
        value: function(path) {
            var _this3 = this;
            var result = (new _deferred.Deferred).resolve().promise();
            if (!path) {
                return result
            }
            var currentPath = "";
            var parts = path.split(PATH_SEPARATOR);
            (0, _iterator.each)(parts, function(_, part) {
                if (currentPath) {
                    currentPath += PATH_SEPARATOR
                }
                currentPath += part;
                var getExpandFunc = function(p) {
                    return function() {
                        return _this3._expandLoadedPath(p)
                    }
                };
                result = result.then(getExpandFunc(currentPath))
            });
            return result
        }
    }, {
        key: "_expandLoadedPath",
        value: function(path) {
            var node = this._filesTreeView._dataAdapter.getNodeByKey(path);
            if (!node.expanded) {
                var deferred = this._loadMap[path];
                if (!deferred) {
                    deferred = new _deferred.Deferred;
                    this._loadMap[path] = deferred;
                    this._filesTreeView.expandItem(path)
                }
                return deferred.promise()
            } else {
                return (new _deferred.Deferred).resolve().promise()
            }
        }
    }, {
        key: "_onFilesTreeViewItemExpanded",
        value: function(e) {
            var path = e.itemData ? e.itemData.relativeName : "";
            var deferred = this._loadMap[path];
            if (deferred) {
                this._loadMap[path] = null;
                deferred.resolve()
            }
        }
    }, {
        key: "_getDefaultOptions",
        value: function() {
            return (0, _extend.extend)(_get(FileManagerFilesTreeView.prototype.__proto__ || Object.getPrototypeOf(FileManagerFilesTreeView.prototype), "_getDefaultOptions", this).call(this), {
                getItems: null,
                onCurrentFolderChanged: null,
                onClick: null
            })
        }
    }, {
        key: "refreshData",
        value: function() {
            this._filesTreeView.option("dataSource", []);
            var currentFolderChanged = "" !== this.getCurrentPath();
            this._initCurrentPathState();
            if (currentFolderChanged) {
                this._raiseCurrentFolderChanged()
            }
        }
    }, {
        key: "setCurrentPath",
        value: function(path) {
            var _this4 = this;
            this._ensurePathExpanded(path).then(function() {
                return _this4._setCurrentLoadedPath(path)
            })
        }
    }, {
        key: "_setCurrentLoadedPath",
        value: function(path) {
            var $node = null;
            var folder = this._rootFolder;
            if (path) {
                var node = this._filesTreeView._dataAdapter.getNodeByKey(path);
                $node = this._filesTreeView._getNodeElement(node);
                folder = node.internalFields.item
            }
            this._filesTreeView.option("focusedElement", $node);
            this._changeCurrentFolder(folder)
        }
    }, {
        key: "getCurrentPath",
        value: function() {
            return this._currentPath
        }
    }, {
        key: "getCurrentFolder",
        value: function() {
            return this._currentFolder
        }
    }]);
    return FileManagerFilesTreeView
}(_ui2.default);
module.exports = FileManagerFilesTreeView;
