/**
 * DevExtreme (ui/scheduler/tooltip_strategies/tooltipStrategyBase.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.TooltipStrategyBase = void 0;
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _button = require("../../button");
var _button2 = _interopRequireDefault(_button);
var _date = require("../../../core/utils/date");
var _date2 = _interopRequireDefault(_date);
var _function_template = require("../../widget/function_template");
var _function_template2 = _interopRequireDefault(_function_template);
var _renderer = require("../../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}
var APPOINTMENT_TOOLTIP_CLASS = "dx-scheduler-appointment-tooltip",
    APPOINTMENT_TOOLTIP_TITLE_CLASS = "dx-scheduler-appointment-tooltip-title",
    APPOINTMENT_TOOLTIP_DATE_CLASS = "dx-scheduler-appointment-tooltip-date",
    APPOINTMENT_TOOLTIP_BUTTONS_CLASS = "dx-scheduler-appointment-tooltip-buttons",
    APPOINTMENT_TOOLTIP_OPEN_BUTTON_CLASS = "dx-scheduler-appointment-tooltip-open-button",
    APPOINTMENT_TOOLTIP_CLOSE_BUTTON_CLASS = "dx-scheduler-appointment-tooltip-close-button",
    APPOINTMENT_TOOLTIP_DELETE_BUTTONS_CLASS = "dx-scheduler-appointment-tooltip-delete-button";
var TooltipStrategyBase = exports.TooltipStrategyBase = function() {
    function TooltipStrategyBase(scheduler) {
        _classCallCheck(this, TooltipStrategyBase);
        this.scheduler = scheduler
    }
    _createClass(TooltipStrategyBase, [{
        key: "show",
        value: function(dataItemList) {}
    }, {
        key: "hide",
        value: function() {}
    }, {
        key: "_getTargetData",
        value: function(data, $appointment) {
            return this.scheduler.fire("getTargetedAppointmentData", data, $appointment)
        }
    }, {
        key: "_renderTemplate",
        value: function(data, currentData, $appointment) {
            var _this = this;
            this._createTemplate(data, currentData, $appointment);
            var template = this.scheduler._getAppointmentTemplate("appointmentTooltipTemplate");
            return new _function_template2.default(function(options) {
                return template.render({
                    model: data,
                    targetedAppointmentData: _this._getTargetData(data, $appointment),
                    container: options.container
                })
            })
        }
    }, {
        key: "_isAppointmentInAllDayPanel",
        value: function(appointmentData) {
            var workSpace = this.scheduler._workSpace,
                itTakesAllDay = this.scheduler.appointmentTakesAllDay(appointmentData);
            return itTakesAllDay && workSpace.supportAllDayRow() && workSpace.option("showAllDayPanel")
        }
    }, {
        key: "_createTemplate",
        value: function(appointmentData, singleAppointmentData, $appointment) {
            var _this2 = this;
            this.scheduler._defaultTemplates.appointmentTooltip = new _function_template2.default(function(options) {
                var $container = (0, _renderer2.default)(options.container),
                    $content = _this2._createTemplateContent(appointmentData, singleAppointmentData, $appointment);
                $content.addClass($container.attr("class"));
                $container.replaceWith($content);
                return $container
            })
        }
    }, {
        key: "_createTemplateContent",
        value: function(appointmentData, singleAppointmentData, $appointment) {
            var $content = (0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_CLASS);
            var isAllDay = this.scheduler.fire("getField", "allDay", appointmentData),
                startDate = this.scheduler.fire("getField", "startDate", singleAppointmentData),
                endDate = this.scheduler.fire("getField", "endDate", singleAppointmentData),
                text = this.scheduler.fire("getField", "text", appointmentData),
                startDateTimeZone = this.scheduler.fire("getField", "startDateTimeZone", appointmentData),
                endDateTimeZone = this.scheduler.fire("getField", "endDateTimeZone", appointmentData);
            startDate = this.scheduler.fire("convertDateByTimezone", startDate, startDateTimeZone);
            endDate = this.scheduler.fire("convertDateByTimezone", endDate, endDateTimeZone);
            var $title = (0, _renderer2.default)("<div>").text(text).addClass(APPOINTMENT_TOOLTIP_TITLE_CLASS).appendTo($content);
            (0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_DATE_CLASS).text(this._formatDate(startDate, endDate, isAllDay)).appendTo($content);
            var $buttons = (0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_BUTTONS_CLASS);
            "top" === this.scheduler.option("_appointmentTooltipButtonsPosition") ? $buttons.prependTo($content) : $buttons.appendTo($content);
            if (this.scheduler.option("_appointmentTooltipCloseButton")) {
                this._createCloseButton().appendTo($buttons)
            }
            if (this.scheduler.option("_useAppointmentColorForTooltip")) {
                this.scheduler.getAppointmentsInstance().notifyObserver("getAppointmentColor", {
                    itemData: appointmentData,
                    groupIndex: $appointment.data("dxAppointmentSettings").groupIndex,
                    callback: function(d) {
                        d.done(function(color) {
                            $title.css("backgroundColor", color);
                            $buttons.css("backgroundColor", color)
                        })
                    }
                })
            }
            if (this.scheduler._editing.allowDeleting) {
                this._createDeleteButton(appointmentData, singleAppointmentData).appendTo($buttons)
            }
            this._createOpenButton(appointmentData, singleAppointmentData).appendTo($buttons);
            return $content
        }
    }, {
        key: "_getFormatType",
        value: function(startDate, endDate, isAllDay) {
            if (isAllDay) {
                return "DATE"
            }
            if ("month" !== this.scheduler.option("currentView") && _date2.default.sameDate(startDate, endDate)) {
                return "TIME"
            }
            return "DATETIME"
        }
    }, {
        key: "_formatDate",
        value: function(startDate, endDate, isAllDay) {
            var result = "";
            this.scheduler.fire("formatDates", {
                startDate: startDate,
                endDate: endDate,
                formatType: this._getFormatType(startDate, endDate, isAllDay),
                callback: function(value) {
                    result = value
                }
            });
            return result
        }
    }, {
        key: "_createDeleteButton",
        value: function(appointmentData, singleAppointmentData) {
            var _this3 = this;
            var button = new _button2.default((0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_DELETE_BUTTONS_CLASS), {
                icon: "trash",
                onClick: function() {
                    var startDate = _this3.scheduler._getStartDate(singleAppointmentData, true);
                    _this3.scheduler._checkRecurringAppointment(appointmentData, singleAppointmentData, startDate, function() {
                        _this3.scheduler.deleteAppointment(appointmentData)
                    }, true);
                    _this3.hide()
                }
            });
            return button.$element()
        }
    }, {
        key: "_createOpenButton",
        value: function(appointmentData, singleAppointmentData) {
            var _this4 = this;
            var allowUpdating = this.scheduler._editing.allowUpdating,
                text = this.scheduler.option("_appointmentTooltipOpenButtonText");
            return new _button2.default((0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_OPEN_BUTTON_CLASS), {
                icon: allowUpdating ? "edit" : this.scheduler.option("_appointmentTooltipOpenButtonIcon"),
                text: text,
                onClick: function() {
                    _this4.scheduler.showAppointmentPopup(appointmentData, false, singleAppointmentData);
                    _this4.hide()
                }
            }).$element()
        }
    }, {
        key: "_createCloseButton",
        value: function() {
            var _this5 = this;
            return new _button2.default((0, _renderer2.default)("<div>").addClass(APPOINTMENT_TOOLTIP_CLOSE_BUTTON_CLASS), {
                icon: "close",
                onClick: function() {
                    _this5.hide()
                }
            }).$element()
        }
    }]);
    return TooltipStrategyBase
}();
