/**
 * DevExtreme (viz/core/annotations.js)
 * Version: 19.1.0 (build 19085-0305)
 * Build date: Tue Mar 26 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.plugins = exports.__test_utils = exports.createAnnotations = void 0;
var _type = require("../../core/utils/type");
var _tooltip = require("../core/tooltip");
var _extend3 = require("../../core/utils/extend");
var _consts = require("../components/consts");
var _utils = require("./utils");

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}
var MOVE_EVENT = _consts.events.mousemove + ".annotations";
var ANNOTATION_DATA = "annotation-data";

function coreAnnotation(options, _draw) {
    return {
        name: options.name,
        x: options.x,
        y: options.y,
        value: options.value,
        argument: options.argument,
        axis: options.axis,
        series: options.series,
        _options: options,
        draw: function(widget, group) {
            this.coords = widget._getAnnotationCoords(this);
            var _coords = this.coords,
                x = _coords.x,
                y = _coords.y;
            (0, _type.isDefined)(x) && (0, _type.isDefined)(y) && _draw.call(this, {
                x: x,
                y: y
            }, widget, group, this._options.image)
        },
        getTooltipFormatObject: function() {
            return (0, _extend3.extend)({}, this._options)
        },
        getTooltipParams: function() {
            var _coords2 = this.coords,
                x = _coords2.x,
                y = _coords2.y;
            return {
                x: x,
                y: y
            }
        }
    }
}

function applyClipPath(elem, widget, pane) {
    (0, _type.isDefined)(pane) && elem.attr({
        "clip-path": widget._getElementsClipRectID(pane)
    })
}

function labelAnnotation(options) {
    return coreAnnotation(options, function(_ref, widget, group) {
        var x = _ref.x,
            y = _ref.y;
        var text = widget._renderer.text(options.label.text, x, y).data(_defineProperty({}, ANNOTATION_DATA, this)).css((0, _utils.patchFontOptions)(options.label.font)).append(group);
        applyClipPath(text, widget, this._pane)
    })
}

function imageAnnotation(options) {
    var _options$image = options.image,
        width = _options$image.width,
        height = _options$image.height,
        url = _options$image.url,
        location = _options$image.location;
    return coreAnnotation(options, function(_ref2, widget, group) {
        var x = _ref2.x,
            y = _ref2.y;
        var image = widget._renderer.image(x - .5 * width, y - .5 * height, width, height, url, location).data(_defineProperty({}, ANNOTATION_DATA, this)).append(group);
        applyClipPath(image, widget, this._pane)
    })
}

function mergeOptions(itemOptions, commonOptions, key) {
    return (0, _extend3.extend)(true, {}, itemOptions, _defineProperty({}, key, commonOptions), _defineProperty({}, key, itemOptions[key]))
}

function createAnnotation(itemOptions, commonOptions) {
    if ((0, _type.isDefined)(itemOptions.image)) {
        return imageAnnotation(mergeOptions(itemOptions, commonOptions.imageOptions, "image"))
    } else {
        if ((0, _type.isDefined)(itemOptions.label)) {
            return labelAnnotation(mergeOptions(itemOptions, commonOptions.labelOptions, "label"))
        }
    }
}
var createAnnotations = exports.createAnnotations = function(options) {
    return options.items.map(function(itemOptions) {
        return createAnnotation(itemOptions, options)
    })
};
var chartPlugin = {
    name: "annotations_chart",
    init: function() {},
    dispose: function() {},
    members: {
        _getAnnotationCoords: function(annotation) {
            var coords = {
                x: annotation.x,
                y: annotation.y
            };
            var argCoordName = this._options.rotated ? "y" : "x";
            var valCoordName = this._options.rotated ? "x" : "y";
            var argument = annotation.argument;
            var value = annotation.value;
            var argAxis = this.getArgumentAxis();
            var axis = this.getValueAxis(annotation.axis);
            var series = void 0;
            annotation._pane = annotation.axis && (0, _type.isDefined)(axis) ? axis.pane : void 0;
            if (annotation.series) {
                series = this.series.filter(function(s) {
                    return s.name === annotation.series
                })[0];
                axis = series && series.getValueAxis();
                (0, _type.isDefined)(axis) && (annotation._pane = axis.pane)
            }
            if (!(0, _type.isDefined)(coords[argCoordName]) && (0, _type.isDefined)(argument)) {
                coords[argCoordName] = argAxis.getTranslator().translate(argument);
                !(0, _type.isDefined)(annotation._pane) && (annotation._pane = argAxis.pane)
            }
            if (!(0, _type.isDefined)(coords[valCoordName]) && (0, _type.isDefined)(value)) {
                coords[valCoordName] = axis && axis.getTranslator().translate(value);
                !(0, _type.isDefined)(annotation._pane) && (0, _type.isDefined)(axis) && (annotation._pane = axis.pane)
            }
            if ((0, _type.isDefined)(coords[argCoordName]) && !(0, _type.isDefined)(coords[valCoordName]) && !(0, _type.isDefined)(value)) {
                if (!(0, _type.isDefined)(axis) && !(0, _type.isDefined)(series)) {
                    coords[valCoordName] = argAxis.getAxisPosition()
                } else {
                    if ((0, _type.isDefined)(axis) && !(0, _type.isDefined)(series)) {
                        coords[valCoordName] = this._argumentAxes.filter(function(a) {
                            return a.pane === axis.pane
                        })[0].getAxisPosition()
                    } else {
                        if ((0, _type.isDefined)(series)) {
                            if (series.checkSeriesViewportCoord(argAxis, coords[argCoordName])) {
                                coords[valCoordName] = series.getSeriesPairCoord(coords[argCoordName], true)
                            }
                            if (!(0, _type.isDefined)(coords[valCoordName])) {
                                coords[valCoordName] = this._argumentAxes.filter(function(a) {
                                    return a.pane === axis.pane
                                })[0].getAxisPosition()
                            }
                        }
                    }
                }
            }
            if (!(0, _type.isDefined)(coords[argCoordName]) && !(0, _type.isDefined)(argument) && (0, _type.isDefined)(coords[valCoordName])) {
                if ((0, _type.isDefined)(axis) && !(0, _type.isDefined)(series)) {
                    coords[argCoordName] = axis.getAxisPosition()
                } else {
                    if ((0, _type.isDefined)(series)) {
                        if (series.checkSeriesViewportCoord(axis, coords[valCoordName])) {
                            coords[argCoordName] = series.getSeriesPairCoord(coords[valCoordName], false)
                        }
                        if (!(0, _type.isDefined)(coords[argCoordName])) {
                            coords[argCoordName] = axis.getAxisPosition()
                        }
                    }
                }
            }
            return coords
        },
        _onMouseMove: function(_ref3) {
            var target = _ref3.target;
            var annotation = target[ANNOTATION_DATA];
            if (!annotation) {
                this._annotations.tooltip.hide();
                return
            }
            this.hideTooltip();
            this.clearHover();
            var tooltipFormatObject = annotation.getTooltipFormatObject(this._annotations.tooltip);
            var coords = annotation.getTooltipParams(this._annotations.tooltip.getLocation()),
                rootOffset = this._renderer.getRootOffset();
            coords.x += rootOffset.left;
            coords.y += rootOffset.top;
            this._annotations.tooltip.show(tooltipFormatObject, coords, {
                target: annotation
            })
        }
    }
};
var corePlugin = {
    name: "annotations_core",
    init: function() {
        this._annotations = {
            items: []
        }
    },
    dispose: function() {
        this._annotationsGroup.linkRemove().linkOff();
        this._renderer.root.off(MOVE_EVENT);
        this._annotations.tooltip && this._annotations.tooltip.dispose()
    },
    extenders: {
        _createHtmlStructure: function() {
            this._annotationsGroup = this._renderer.g().attr({
                "class": this._rootClassPrefix + "-annotations"
            }).linkOn(this._renderer.root, "annotations").linkAppend()
        },
        _renderExtraElements: function() {
            var _this = this;
            this._annotationsGroup.clear();
            this._annotations.items.forEach(function(item) {
                return item.draw(_this, _this._annotationsGroup)
            })
        }
    },
    members: {
        _buildAnnotations: function() {
            this._annotations.items = [];
            var options = this._getOption("annotations");
            if (!options || !options.items) {
                return
            }
            this._annotations.tooltip = new _tooltip.Tooltip({
                cssClass: "dxc-tooltip",
                eventTrigger: this._eventTrigger,
                widgetRoot: this.element()
            });
            this._annotations.tooltip.setRendererOptions(this._getRendererOptions());
            var tooltipOptions = (0, _extend3.extend)({}, this._themeManager.getOptions("tooltip"), {
                enabled: false
            });
            if (options.customizeTooltip) {
                tooltipOptions.customizeTooltip = options.customizeTooltip;
                tooltipOptions.enabled = true
            }
            this._annotations.tooltip.update(tooltipOptions);
            this._annotations.items = createAnnotations(options);
            this._renderer.root.on(MOVE_EVENT, this._onMouseMove.bind(this))
        },
        _getAnnotationCoords: function() {
            return {}
        }
    },
    customize: function(constructor) {
        constructor.addChange({
            code: "ANNOTATIONS",
            handler: function() {
                this._buildAnnotations();
                this._change(["FORCE_RENDER"])
            },
            isThemeDependent: true,
            isOptionChange: true,
            option: "annotations"
        })
    },
    fontFields: ["annotations.labelOptions.font"]
};
var plugins = exports.plugins = {
    core: corePlugin,
    chart: chartPlugin
};
